#include "myconsole.h"

#define LOGFILE "C:\\wss_log.txt"

static const WORD MAX_CONSOLE_LINES = 500;

/* Create a Console inside a GUI process, and redirect
 * the standard I/O function (printf, scanf, etc)
 */
void RedirectIOToConsole()
{
    int hConHandle;
    long lStdHandle;

    CONSOLE_SCREEN_BUFFER_INFO coninfo;
    FILE *fp;

    // Allocate a console for this app
    AllocConsole();

    // Set the screen buffer to be big enough to let us scroll text
    GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), &coninfo);
    coninfo.dwSize.Y = MAX_CONSOLE_LINES;
    SetConsoleScreenBufferSize(GetStdHandle(STD_OUTPUT_HANDLE), coninfo.dwSize);

    // Redirect unbuffered STDOUT to the console
    lStdHandle = (long)GetStdHandle(STD_OUTPUT_HANDLE);
    hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
    fp = _fdopen( hConHandle, "w" );
    *stdout = *fp;
    setvbuf( stdout, NULL, _IONBF, 0 );

    // Redirect unbuffered STDIN to the console
    lStdHandle = (long)GetStdHandle(STD_INPUT_HANDLE);
    hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
    fp = _fdopen( hConHandle, "r" );
    *stdin = *fp;
    setvbuf( stdin, NULL, _IONBF, 0 );

    // Redirect unbuffered STDERR to the console
    lStdHandle = (long)GetStdHandle(STD_ERROR_HANDLE);
    hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
    fp = _fdopen( hConHandle, "w" );
    *stderr = *fp;
    setvbuf( stderr, NULL, _IONBF, 0 );

//End of File

}

long Error(const char* msg, ...)
{
    va_list fmtargs;
    char buffer[1024];

    // Process arguments
    va_start(fmtargs,msg);
    vsnprintf(buffer, sizeof(buffer)-1, msg, fmtargs);
    va_end(fmtargs);

 #ifdef LOGFILE
    // Log the message to a file
    FILE *logfile;
    logfile = fopen(LOGFILE,"a+"); /* apend file (add text to a file or create a file if it does not exist.*/
    if (logfile == NULL)
        return errno;
    fprintf(logfile, "%s", buffer);
    fclose(logfile);
 #endif

    // Print out the message
    printf("%s", buffer);

    return ERROR_SUCCESS;
}

